@@ -8,6 +8,8 @@ from .dlms_templates import DLMS_TEMPLATES
 from pytz import timezone
 from copy import copy
 
+from datetime import datetime
+
 TZ = timezone('Europe/Madrid')
 
 
@@ -33,6 +35,19 @@ def datetimetoprime(dt):
     return dt_str
 
 
+def datetohexprime(dt):
+    """
+    Converts a date to a hexadecimal prime date string
+    """
+    year = dt.year
+    month = dt.month
+    day = dt.day
+
+    date_string = '{0:04x}{1:02x}{2:02x}'.format(year, month, day).upper()
+
+    return date_string
+
+
 def name2octet(txt):
     octet_str = ''
     for caracter in '{: >6}'.format(txt):
@@ -47,8 +62,19 @@ def octet2name(txt):
     return name
 
 
-def octet2hour(txt):
-    return int(txt[0:2], 16)
+def octet2number(txt):
+    return int(txt, 16)
+
+
+def octet2date(txt):
+    year = octet2number(txt[0:4])
+    month = octet2number(txt[4:6])
+    day = octet2number(txt[6:8])
+    hour = octet2number(txt[8:10])
+    minute = octet2number(txt[10:12])
+    second = octet2number(txt[12:14])
+
+    return datetime.strptime('{}-{}-{} {}:{}:{}'.format(year, month, day, hour, minute, second), '%Y-%m-%d %H:%M:%S')
 
 
 class PrimeTemplates:
@@ -59,10 +85,10 @@ class PrimeTemplates:
     def get_available_templates(self, origin=None, template_type=None):
         template_list = []
         for name, contract in self.templates.items():
-            if origin is not None and template_type is not None:
-                if (origin is not None and contract['origin'] != origin) or (
-                        template_type is not None and contract['type'] != template_type):
-                    continue
+            if origin is not None and contract['origin'] != origin:
+                continue
+            if template_type is not None and contract['category'] != template_type:
+                continue
 
             template_list.append((name, contract['description'], contract['origin']))
 
@@ -85,3 +111,20 @@ class DLMSTemplates(PrimeTemplates):
 
     def __init__(self):
         self.templates = DLMS_TEMPLATES
+
+    def generate_cycle_file(self, template_name, meters_name):
+        elements = self.get_template(template_name)['data']
+
+        xml = '<cycles>\n<cycle name="Ciclo_{}_raw" period="1" immediate="true" repeat="1" priority="1">\n'.format(
+            template_name)
+
+        for meter_name in meters_name:
+            xml += '<device sn="{}"/>\n'.format(meter_name)
+
+        for element in elements:
+            xml += '<set obis="{}" class="{}" element="{}">{}</set>\n'.format(
+                element['obis'], element['class'], element['element'], element['data'])
+
+        xml += '</cycle>\n</cycles>'
+
+        return xml
